/**
 * @file Layout
 * @description 页面布局，支持左边栏、顶部、内容区域布局。
 * @author fex
 *
 * @param 参数说明：
 * * children 会渲染在内容区。
 * * header 头部区域
 * * aside 边栏
 * * asideWide 边栏是否加宽
 * * asideClassName 边栏附加样式class
 * * footer 页脚
 * * folder 是否收起边栏
 * * asideFixed 边栏是否为固定模式，如果是会用 position:fixed 来定位.
 * * className 附件的样式名
 * * contentClassName 内容区域附加样式名称
 */
import React from 'react';
import { ClassNamesFn } from '../theme';
interface LayoutProps {
    id: string;
    header?: boolean | React.ReactNode;
    aside?: boolean | React.ReactNode;
    asideClassName: string;
    folded?: boolean;
    asideFixed: boolean;
    headerFixed: boolean;
    className?: string;
    contentClassName?: string;
    footer: boolean | React.ReactNode;
    asideWide: boolean;
    offScreen: boolean;
    classPrefix: string;
    classnames: ClassNamesFn;
    size?: 'sm' | 'base' | 'md' | 'lg';
}
export declare class Layout extends React.Component<LayoutProps, any> {
    static defaultProps: {
        asideFixed: boolean;
        asideClassName: string;
        headerFixed: boolean;
        offScreen: boolean;
        footer: boolean;
    };
    render(): JSX.Element;
}
declare const _default: React.ComponentClass<Pick<Pick<LayoutProps, "aside" | "header" | "size" | "id" | "className" | "classPrefix" | "classnames" | "folded" | "contentClassName" | "asideWide"> & Partial<Pick<LayoutProps, "footer" | "asideClassName" | "asideFixed" | "headerFixed" | "offScreen">> & Partial<Pick<{
    asideFixed: boolean;
    asideClassName: string;
    headerFixed: boolean;
    offScreen: boolean;
    footer: boolean;
}, never>>, "aside" | "footer" | "header" | "size" | "id" | "className" | "asideClassName" | "folded" | "asideFixed" | "headerFixed" | "contentClassName" | "asideWide" | "offScreen"> & {
    theme?: string | undefined;
    classPrefix?: string | undefined;
    classnames?: ClassNamesFn | undefined;
}, any> & {
    ComposedComponent: typeof Layout;
};
export default _default;
